﻿using System.Collections.Generic;
using System.Web.Mvc;
using WebApplication1.Models;

namespace WebApplication1.Controllers
{
  // please have also look on PersonControllerTest class, where zou can find tests for this controller

  // HandleError attribute provides error handling => when exception occurs, Error view is shown
  [HandleError]

  // Every action/method requires authorization
  //[Authorize]

  // No action/method requires authorization
  //[AllowAnonymous]
  public class PersonController : Controller
  {
    static PersonController()
    {
      Persons.Add(new Person()
      {
        Name = "Jaroslav Nečas",
        Id = 1,
        Sex = Sex.Male
      });
    }

    private static readonly List<Person> Persons = new List<Person>();

    // GET: Person
    public ActionResult Index()
    {
      return View(Persons);
    }

    // GET: Person/Details/5
    public ActionResult Details(int id)
    {
      var person = GetPersonById(id);
      return View(person);
    }

    // GET: Person/Create
    public ActionResult Create()
    {
      var person = new Person();
      TempData["Data"] = PrepareSexData(person);
      return View(person);
    }

    // POST: Person/Create
    [HttpPost]
    public ActionResult Create(FormCollection collection)
    {
      var person = new Person();
      TryUpdateModel(person);
      TryValidateModel(person);
      
      //throw new Exception();

      if (!ModelState.IsValid)
      {
        return View(person);
      }

      Persons.Add(person);

      return RedirectToAction("Index");
    }

    // GET: Person/Edit/5
    public ActionResult Edit(int id)
    {
      var person = GetPersonById(id);
      TempData["Data"] = PrepareSexData(person);
      return View(person);
    }

    // POST: Person/Edit/5
    [HttpPost]
    public ActionResult Edit(int id, FormCollection collection)
    {
      var person = GetPersonById(id);
      TryUpdateModel(person);
      TryValidateModel(person);

      if (!ModelState.IsValid)
      {
        return View(person);
      }

      return RedirectToAction("Index");
    }

    // GET: Person/Delete/5
    public ActionResult Delete(int id)
    {
      var person = GetPersonById(id);
      return View(person);
    }

    // POST: Person/Delete/5
    [HttpPost]
    public ActionResult Delete(int id, FormCollection collection)
    {
      var person = GetPersonById(id);
      Persons.Remove(person);

      return RedirectToAction("Index");
    }

    private static Person GetPersonById(int id)
    {
      return Persons.Find(i => i.Id == id);
    }

    private List<SelectListItem> PrepareSexData(Person person)
    {
      var result = new List<SelectListItem>();
      result.Add(new SelectListItem()
      {
        Selected = person.Sex == Sex.Unknown,
        Value = Sex.Unknown.ToString(),
        Text = Sex.Unknown.ToString()
      });

      result.Add(new SelectListItem()
      {
        Selected = person.Sex == Sex.Male,
        Value = Sex.Male.ToString(),
        Text = Sex.Male.ToString()
      });

      result.Add(new SelectListItem()
      {
        Selected = person.Sex == Sex.Female,
        Value = Sex.Female.ToString(),
        Text = Sex.Female.ToString()
      });

      return result;
    }
  }
}
